Module: AppMap

Defined in:
lib/appmap/hook.rb,
lib/appmap/open.rb,
lib/appmap/util.rb,
lib/appmap/agent.rb,
lib/appmap/event.rb,
lib/appmap/rspec.rb,
lib/appmap/trace.rb,
lib/appmap/config.rb,
lib/appmap/handler.rb,
lib/appmap/railtie.rb,
lib/appmap/version.rb,
lib/appmap/cucumber.rb,
lib/appmap/metadata.rb,
lib/appmap/minitest.rb,
lib/appmap/node_cli.rb,
lib/appmap/class_map.rb,
lib/appmap/depends/api.rb,
lib/appmap/hook/method.rb,
lib/appmap/depends/util.rb,
lib/appmap/command/index.rb,
lib/appmap/command_error.rb,
lib/appmap/swagger/stable.rb,
lib/appmap/command/inspect.rb,
lib/appmap/service/guesser.rb,
lib/appmap/depends/node_cli.rb,
lib/appmap/hook/method/ruby2.rb,
lib/appmap/hook/method/ruby3.rb,
lib/appmap/depends/rake_tasks.rb,
lib/appmap/swagger/rake_tasks.rb,
lib/appmap/depends/test_runner.rb,
lib/appmap/handler/eval_handler.rb,
lib/appmap/depends/configuration.rb,
lib/appmap/swagger/configuration.rb,
lib/appmap/handler/rails/template.rb,
lib/appmap/service/config_analyzer.rb,
lib/appmap/command/agent_setup/init.rb,
lib/appmap/handler/function_handler.rb,
lib/appmap/handler/net_http_handler.rb,
lib/appmap/handler/open_ssl_handler.rb,
lib/appmap/handler/rails/sql_handler.rb,
lib/appmap/command/agent_setup/status.rb,
lib/appmap/depends/test_file_inspector.rb,
lib/appmap/middleware/remote_recording.rb,
lib/appmap/service/validator/violation.rb,
lib/appmap/command/agent_setup/validate.rb,
lib/appmap/handler/marshal_load_handler.rb,
lib/appmap/handler/rails/render_handler.rb,
lib/appmap/handler/rails/request_handler.rb,
lib/appmap/service/test_command_provider.rb,
lib/appmap/swagger/markdown_descriptions.rb,
lib/appmap/service/test_framework_detector.rb,
lib/appmap/service/validator/config_validator.rb,
lib/appmap/service/integration_test_path_finder.rb,
ext/appmap/appmap.c

Defined Under Namespace

Modules: Command, Cucumber, Depends, Event, Handler, Metadata, Middleware, Minitest, ModuleMethods, ObjectMethods, RSpec, Service, Swagger, Trace, Util Classes: ClassMap, CommandError, Config, Hook, NodeCLI, Open, OpenStruct, Railtie, StackPrinter, Tracer

Constant Summary collapse

URL =
'https://github.com/applandinc/appmap-ruby'
VERSION =
'0.87.0'
APPMAP_FORMAT_VERSION =
'1.7.0'
SUPPORTED_RUBY_VERSIONS =
%w[2.5 2.6 2.7 3.0 3.1].freeze
DEFAULT_APPMAP_DIR =
'tmp/appmap'.freeze
DEFAULT_CONFIG_FILE_PATH =
'appmap.yml'.freeze

Class Method Summary collapse

Class Method Details

.caller_binding(level) ⇒ Object



98
99
100
101
102
# File 'ext/appmap/appmap.c', line 98

static VALUE
am_previous_bindings(VALUE self, VALUE level)
{
  return rb_debug_inspector_open(bindings_callback, (void *) level);
}

.class_map(methods) ⇒ Object

Builds a class map from a config and a list of Ruby methods.



99
100
101
# File 'lib/appmap/agent.rb', line 99

def class_map(methods)
  ClassMap.build_from_methods(methods)
end

.configurationObject

Gets the configuration. If there is no configuration, the default configuration is initialized.



20
21
22
# File 'lib/appmap/agent.rb', line 20

def configuration
  @configuration ||= initialize_configuration
end

.configuration=(config) ⇒ Object

Sets the configuration. This is only expected to happen once per Ruby process.



26
27
28
29
30
# File 'lib/appmap/agent.rb', line 26

def configuration=(config)
  warn 'AppMap is already configured' if @configuration && config

  @configuration = config
end

.default_config_file_pathObject



32
33
34
# File 'lib/appmap/agent.rb', line 32

def default_config_file_path
  ENV['APPMAP_CONFIG_FILE'] || 'appmap.yml'
end

.detect_metadataObject

Returns default metadata detected from the Ruby system and from the filesystem.



105
106
107
108
# File 'lib/appmap/agent.rb', line 105

def 
   ||= .detect.freeze
  Util.deep_dup()
end

.explain_queries?Boolean

Returns:

  • (Boolean)


114
115
116
# File 'lib/appmap/agent.rb', line 114

def explain_queries?
  ENV['APPMAP_EXPLAIN_QUERIES'] == 'true'
end

.info(msg) ⇒ Object



52
53
54
55
56
57
58
# File 'lib/appmap/agent.rb', line 52

def info(msg)
  if defined?(::Rails) && defined?(::Rails.logger)
    ::Rails.logger.info msg
  else
    warn msg
  end
end

.initialize_configuration(config_file_path = default_config_file_path) ⇒ Object

Configures AppMap for recording. Default behavior is to configure from APPMAP_CONFIG_FILE, or 'appmap.yml'. If no config file is available, a configuration will be automatically generated and used - and the user is prompted to create the config file.

This method also activates the code hooks which record function calls as trace events. Call this function before the program code is loaded by the Ruby VM, otherwise the load events won't be seen and the hooks won't activate.



44
45
46
47
48
49
50
# File 'lib/appmap/agent.rb', line 44

def initialize_configuration(config_file_path = default_config_file_path)
  Util.startup_message "Configuring AppMap from path #{config_file_path}"
  Config.load_from_file(config_file_path).tap do |configuration|
    self.configuration = configuration
    Hook.new(configuration).enable
  end
end

.open(appmap = nil, &block) ⇒ Object

Uploads an AppMap to the AppLand website and displays it.



93
94
95
96
# File 'lib/appmap/agent.rb', line 93

def open(appmap = nil, &block)
  appmap ||= AppMap.record(&block)
  AppMap::Open.new(appmap).perform
end

.parameter_schema?Boolean

Returns:

  • (Boolean)


110
111
112
# File 'lib/appmap/agent.rb', line 110

def parameter_schema?
  ENV['APPMAP_PARAMETER_SCHEMA'] == 'true'
end

.recordObject

Records the events which occur while processing a block, and returns an AppMap as a Hash.



73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
# File 'lib/appmap/agent.rb', line 73

def record
  tracer = tracing.trace
  begin
    yield
  ensure
    tracing.delete(tracer)
  end

  events = [].tap do |event_list|
    event_list << tracer.next_event.to_h while tracer.event?
  end
  {
    'version' => AppMap::APPMAP_FORMAT_VERSION,
    'metadata' => ,
    'classMap' => class_map(tracer.event_methods),
    'events' => events
  }
end

.tracingObject

Used to start tracing, stop tracing, and record events.



67
68
69
# File 'lib/appmap/agent.rb', line 67

def tracing
  @tracing ||= Trace::Tracing.new
end

.tracing_enabled?Boolean

Returns:

  • (Boolean)


60
61
62
63
64
# File 'lib/appmap/agent.rb', line 60

def tracing_enabled?
  return false unless @tracing

  @tracing.enabled?
end