Class: AppMap::Event::MethodEvent

Inherits:
MethodEventStruct show all
Defined in:
lib/appmap/event.rb

Constant Summary collapse

LIMIT =
100

Instance Attribute Summary

Attributes inherited from MethodEventStruct

#event, #id, #thread_id

Class Method Summary collapse

Instance Method Summary collapse

Class Method Details

.best_class_name(value) ⇒ Object

Heuristic for dynamically defined class whose name can be nil



74
75
76
77
78
79
80
# File 'lib/appmap/event.rb', line 74

def best_class_name(value)
  value_cls = value.class
  while value_cls.name.nil?
    value_cls = value_cls.superclass
  end
  value_cls.name
end

.build_from_invocation(event_type, event:) ⇒ Object



26
27
28
29
30
# File 'lib/appmap/event.rb', line 26

def build_from_invocation(event_type, event:)
  event.id = AppMap::Event.next_id_counter
  event.event = event_type
  event.thread_id = Thread.current.object_id
end

.custom_display_string(value) ⇒ Object



86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
# File 'lib/appmap/event.rb', line 86

def custom_display_string(value)
  case value
  when NilClass, TrueClass, FalseClass, Numeric, Time, Date
    value.to_s
  when String
    value[0...LIMIT].encode('utf-8', invalid: :replace, undef: :replace, replace: '_')
  when File
    "#{value.class}[path=#{value.path}]"
  when Net::HTTP
    "#{value.class}[#{value.address}:#{value.port}]"
  when Net::HTTPGenericRequest
    "#{value.class}[#{value.method} #{value.path}]"
  end
rescue StandardError
  nil
end

.default_display_string(value) ⇒ Object



103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
# File 'lib/appmap/event.rb', line 103

def default_display_string(value)
  return nil if ENV['APPMAP_OBJECT_STRING'] == 'false'

  last_resort_string = lambda do
    warn "AppMap encountered an error inspecting a #{value.class.name}: #{$!.message}"
    '*Error inspecting variable*'
  end

  begin
    value.to_s
  rescue NoMethodError
    begin
      value.inspect
    rescue
      last_resort_string.call
    end
  rescue WeakRef::RefError
    nil
  rescue
    last_resort_string.call
  end
end

.display_string(value) ⇒ Object

Gets a display string for a value. This is not meant to be a machine deserializable value.



33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
# File 'lib/appmap/event.rb', line 33

def display_string(value)
  return nil if value.equal?(nil)

  # With setting APPMAP_PROFILE_DISPLAY_STRING, stringifying this class is shown to take 9 seconds(!) of a 17 second test run.
  return nil if best_class_name(value) == 'ActiveSupport::Callbacks::Filters::Environment'

  if @times.nil? && ENV['APPMAP_PROFILE_DISPLAY_STRING'] == 'true'
    @times = Hash.new {|memo,key| memo[key] = 0}
    Thread.new do
      sleep 0.5
      while true
        warn @times.to_a.sort{|a,b| b[1] <=> a[1]}[0..9].join("\n")
        sleep 3
      end
    end
  end

  start = Time.now
  value_string = custom_display_string(value) || default_display_string(value)

  if @times
    elapsed = Time.now - start
    @times[best_class_name(value)] += elapsed
  end

  encode_display_string(value_string)
end

.encode_display_string(value) ⇒ Object



82
83
84
# File 'lib/appmap/event.rb', line 82

def encode_display_string(value)
  (value||'')[0...LIMIT].encode('utf-8', invalid: :replace, undef: :replace, replace: '_')
end

.object_properties(hash_like) ⇒ Object



61
62
63
64
65
66
67
68
69
70
71
# File 'lib/appmap/event.rb', line 61

def object_properties(hash_like)
  hash = hash_like.to_h
  hash.keys.map do |key|
    {
      name: key,
      class: hash[key].class.name,
    }
  end
rescue
  nil
end

Instance Method Details

#ready?Boolean

An event may be partially constructed, and then completed at a later time. When the event is only partially constructed, it's not ready for serialization to the AppMap file.

Returns:

  • (Boolean)

    false until the event is fully constructed and available.



131
132
133
# File 'lib/appmap/event.rb', line 131

def ready?
  true
end